from datetime import datetime, date

__all__ = ("RedefinedField", "ListField", "DecimalField", "IntegerField", "StringField", "DateField")

class Type(object):
    """Type objects will hold onto values that move between cobol copybooks and django models"""

    def __init__(self, value, length, order, path):
        self.value = value
        self.length = length
        self.order = order
        self.path = path
        self.mapped = False

    def strip_self_from_record(self, record):
        """
        strip the value from the beginning of the passed in record and return the new record so the next field
        can strip itself from the beginning of it.
        """
        self.value = record[0:self.length]
        return record[self.length:]

    def populate_from_dict(self, model_dict):
        """
        find the mapping needed for this object in a passed in dictionary
        """
        if self.mapped:
            return

        value = model_dict
        if model_dict and self.path:
            nodes = self.path.split('.')
            for node in nodes:
                if node in value:
                    value = value[node]
                else:
                    value = None
                    break
            else:
                self.mapped = True

            self.value = value

    def number_format(self):
        return "%0" + str(self.length) + "d"

    def char_format(self):
        return "%-" + str(self.length) + "s"

    def trunc_number(self):
        if not self.value:
            self.value = 0
        elif not str(self.value).strip():
            self.value = 0
        try:
            return int(str(self.value)[0:self.trunc_length])
        except ValueError:
            return float(str(self.value)[0:self.trunc_length])

    def trunc_char(self):
        return str(self.value)[0:self.length]

    @property
    def trunc_length(self):
        return self.length

class Date(Type):

    def __init__(self, format, *args, **kwargs):
        super(Date, self).__init__(*args, **kwargs)
        self.format = format
        self.length = kwargs.get('length')

    def to_model(self):
        return datetime.strptime(self.value, self.format)

    def to_copybook(self):
        if isinstance(self.value, (datetime, date)):
            return self.value.strftime(self.format)
        else:
            return self.value or " " * self.length

class Number(Type):
    """Number is used to know when to return a string for COBOL or an actual number for Python"""

    @property
    def format(self):
        return self.number_format()

    def _trunc(self):
        """we need to make sure that we never exceed the max space allowed by the field"""
        return self.trunc_number()

    def __str__(self):
        return self.format % self._trunc()

    def __int__(self):
        return self._trunc()

    def to_model(self):
        return int(self)

    def to_copybook(self):
        return str(self)

class SignedNumber(Number):
    @property
    def format(self):
        return "%+0" + str(self.length) + "d"

    @property
    def trunc_length(self):
        if int(self.value) > 1:
            return self.length - 1
        return self.length

class Decimal(Type):

    def __float__(self):
        if not self.value:
            self.value = 0
        return float(self.value)

    def __str__(self):
        return str(self.value)

    def to_model(self):
        "return an int value to be used by python code"
        return float(self)

    def to_copybook(self):
        "return a string value to be sued by cobol"
        return str(self)

class SignedDecimal(Decimal):

    @property
    def format(self):
        return "%+0" + str(self.length) + ".2f"

    def __str__(self):
        if self.value is None:
            return ' ' * self.length
        return str(self.format % self._trunc())

    @property
    def trunc_length(self):
        if int(self.value) > 1:
            return self.length - 1
        return self.length

    def _trunc(self):
        return float(str(self.value)[-self.trunc_length:])

class Char(Type):
    """Char is used to store character values to pass between python and cobol"""

    @property
    def format(self):
        """format the string padding with spaces to the right"""
        return self.char_format()

    def __str__(self):
        """apply formating and truncate any additional"""
        if not self.value:
            self.value = ""
        return self.format % str(self.value)[0:self.length]

    def to_model(self):
        return str(self)

    def to_copybook(self):
        return str(self)

class NestedCopybook(Type):

    def __init__(self, selector, *args):
        super(NestedCopybook, self).__init__(*args)
        self.selector = selector

    def to_copybook(self):
        return self.value.to_record()

    def strip_self_from_record(self, record):
        """
        strip the value from the beginning of the passed in record and return the new record so the next field
        can strip itself from the beginning of it.
        """
        tmp_rec = record[0:self.length]
        copybook_class = self.selector(tmp_rec)
        self.value = copybook_class.from_record(tmp_rec)
        return record[self.length:]

class List(Type):
    """List is used to store occurances of nested coypbooks"""

    def __init__(self, value, length, order, path, copybook):
        super(List, self).__init__(value, length, order, path)
        self.copybook = copybook

    def populate_from_dict(self, model_list):
        super(List, self).populate_from_dict(model_list)
        values = []
        for model_data in self.value:
            values.append(self.copybook.from_dict(model_data))
        self.value = values

    def to_copybook(self):
        values = [val for val in self.value]
        while len(values) < self.length:
            values.append(self.copybook(to_copybook=True))

        return ''.join([val.to_record() for val in values])

    def to_model(self):
        return self.value

    def strip_self_from_record(self, record):
        """
        strip the value from the beginning of the passed in record and return the new record so the next field
        can strip itself from the beginning of it.
        """
        self.value = []
        for _ in range(self.length):
            copybook = self.copybook(to_model=True)
            record_length = copybook.get_record_length()
            copybook.populate_from_record(record[0:record_length])
            self.value.append(copybook)
            record = record[record_length:]
        return record

class Field(object):
    """
    Field objects are Descriptors that Copybook objects will use to define their fields
    Field objects use the Desciptor interface to add fields like __field_1, __field_2, __field_3
    to the copybook which actually contain the real values that will be passed back and forth from
    python to cobol
    """
    order = 0
    length = 0
    default_value = ''
    default_class = lambda value, length, order, path: None
    to_model = False
    to_copybook = False

    def __init__(self, length=0, order=1, path=None, default=None):
        self.length = length
        self.order = order
        self.path = path
        self.default_value = default or self.default_value

    def build_name(self, order):
        """create hidden name on copybook instance"""
        return "__field_" + str(order)

    def default(self, instance=None):
        """
        return an instance of the default Type class that contains a default value
        for example a Char instance containing an empty string as a value
        """
        return self.default_class(self.default_value, self.length, self.order, self.path)

    def __get__(self, instance, typ):
        field = self.build_name(self.order)
        obj = getattr(instance, field, self.default(instance))
        setattr(instance, field, obj)

        if instance.to_model:
            return obj.to_model()
        elif instance.to_copybook:
            return obj.to_copybook()
        else:
            return obj

    def __set__(self, instance, value):
        field = self.build_name(self.order)
        obj = getattr(instance, field, self.default(instance))
        obj.value = value
        setattr(instance, field, obj)

class DateField(Field):
    default_class = Date
    default_value = ""

    def __init__(self, format, *args, **kwargs):
        super(DateField, self).__init__(*args, **kwargs)
        self.format = format

    def default(self, instance=None):
        """
        return an instance of the default Type class that contains a default value
        for example a Char instance containing an empty string as a value
        """
        return self.default_class(value=self.default_value, length=self.length, order=self.order,
                                  path=self.path, format=self.format)

class StringField(Field):
    """String field that will format strings to/from cobol copybook and django models"""

    default_class = Char
    default_value = ' '

class IntegerField(Field):

    default_class = Number
    default_value = 0

class SignedIntegerField(Field):
    default_class = SignedNumber
    default_value = 0

class SignedDecimalField(Field):
    default_class = SignedDecimal
    default_value = 0

class DecimalField(Field):
    default_class = Decimal
    default_value = 0

class ListField(Field):
    """
    list field will hold onto a a list of copybook objects - this is to be used for copybook
    fields that 'occur' more than once
    """
    default_class = List
    default_value = []

    def __init__(self, length=0, order=1, path=None, copybook=None):
        super(ListField, self).__init__(length, order, path)
        self.copybook = copybook

    def default(self, instance=None):
        """
        return an instance of the default Type class that contains a default value
        for example a Char instance containing an empty string as a value
        """
        return self.default_class(self.default_value, self.length, self.order, self.path, self.copybook)

class RedefinedField(Field):
    """
    a redefined field is a chunk of a file that can be defined in multiple ways
    """

    default_class = NestedCopybook
    default_value = ""

    def __init__(self, length=0, order=1, path=None, select_func=''):
        super(RedefinedField, self).__init__(length, order, path)
        self.select_copybook_func = select_func

    def default(self, instance):
        """
        return an instance of the default Type class that contains a default value
        for example a Char instance containing an empty string as a value
        """
        selector = getattr(instance, self.select_copybook_func)
        return self.default_class(selector, self.default_value, self.length, self.order, self.path)

class NumberOrChar(Type):
    """NumberOrChar is used when we need to treat numeric values as numbers, but also allow alphabetic values."""

    def __init__(self, value, length, order, path, x):
        self.value = value
        self.length = length
        self.order = order
        self.path = path
        self.mapped = False
        self.x = x

    @property
    def format(self):
        return self.check_numeric(self.number_format, self.char_format)

    def _trunc(self):
        return self.check_numeric(self.trunc_number, self.trunc_char)

    def __str__(self):
        return self.format % self._trunc()

    def __int__(self):
        return self._trunc()

    def to_model(self):
        return self.value

    def to_copybook(self):
        return str(self)

    def check_numeric(self, num_call, alpha_call):
        try:
            float(self.value)
        except ValueError:
            return alpha_call()
        else:
            return num_call()

class NumberOrStringField(Field):
    default_class = NumberOrChar
    default_value = '?'

    def default(self, instance=None):
        """
        return an instance of the default Type class that contains a default value
        for example a Char instance containing an empty string as a value
        """
        return self.default_class(self.default_value * self.length, self.length, self.order, self.path, self)



"""
This module handles converting Django models into objects that can be easily ported to other formats (ie: copybooks)

NOTE:  there is a lot of code in here that heavily relies on Django's internals - this could be proplematic when
updating to different versions of Django

"""

import re
from django.db.models.fields.related import RelatedField, ForeignKey

class DictTools(object):

    @staticmethod
    def merge(d_1, d_2):
        result = d_1
        for k, v in d_2.items():
            if k in result:
                if type(v) == dict:
                    result[k] = DictTools.merge(result[k], v)
                else:
                    result[k] = v
            else:
                result[k] = v
        return result

    @staticmethod
    def fold(dict_list):
        d_1 = dict_list[0]
        for d_2 in dict_list[1:]:
            DictTools.merge(d_1, d_2)
        return d_1

def model_to_dict(instance):
    "Returns a dictionary containing field names and values for the given django model instance"
    data = {}

    for field in instance._meta.fields:
        if not isinstance(field, RelatedField):
            value = getattr(instance, field.name, None)
            data[field.name] = value

    return data

def turn_off_flags(func):
    "This allows the copybook to get around the normal rules of accessing it's fields"
    def wrapper(self, *args, **kwargs):
        "wrapper func"
        tmp_to_model = self.to_model
        tmp_to_copybook = self.to_copybook
        self.to_model = False
        self.to_copybook = False
        return_value = func(self, *args, **kwargs)
        self.to_model = tmp_to_model
        self.to_copybook = tmp_to_copybook
        return return_value
    return wrapper

class Copybook(object):
    """
    this class will take care of formatting django models into cobol copybook friendly strings
    """
    def __init__(self, to_copybook=False, to_model=False):
        self.to_model = to_model
        self.to_copybook = to_copybook
        self._fields = []

        if not any((self.to_copybook, self.to_model)):
            raise ValueError("must choose!!!")

    def get_record_length(self):
        "get the total length of all fields"
        fields = self.get_fields()
        length = 0
        for field in fields:
            length += field.length
        return length

    def populate_from_record(self, record):
        "populate this copybook's fields with the given record"
        fields = self.get_fields()
        tmp_record = record
        for field in fields:
            tmp_record = field.strip_self_from_record(tmp_record)

    def populate_from_dict(self, model_dict):
        "populate this copybook's fields from a given dictionary"
        fields = self.get_fields()
        for field in fields:
            field.populate_from_dict(model_dict)

    def populate_from_model(self, model):
        "populate this copybook's fields from a given model - by first converting the model to a dictionary"
        self.populate_from_dict(model_to_dict(model))

    def populate_from_model_list(self, instance_list):
        "populate this copybook's fields from a given list of models - by calling populate_from_model for each model"
        for model in instance_list:
            self.populate_from_model(model)

    def populate_from_model_dict(self, instance_dict):
        """
        populate this copybook's fields from a passed in dictionary of models by converting the dictionary of models
        into a single diconary by converting each model into a diconary and replacing the model instance with a
        dictionary.

        tmp_dict is used so the original instance_dict is left unchanged
        """
        tmp_dict = {}
        for key in instance_dict:
            tmp_dict[key] = model_to_dict(instance_dict[key])
        self.populate_from_dict(tmp_dict)

    def to_record(self):
        "convert this copybook instance into a single record string"
        fields = self.get_fields()
        record = []
        for field in fields:
            record.append(field.to_copybook())
        return ''.join(record)

    def get_fields(self):
        "return the list of fields in this coypbook"
        if not self._fields:
            self._fields = self._build_field_list()
        return self._fields

    def to_dict(self):
        "returns a dictionary for each field with correct paths in the dictionary for each field"
        def other(path, last_value):
            nodes = path.split('.')
            value = nodes.pop(0)
            if value:
                return {value: other('.'.join(nodes), last_value)}
            else:
                return last_value

        def build_field_paths(copybook):
            field_paths = []
            for field in copybook.get_fields():
                if isinstance(field.value, list):
                    values = []
                    for value in field.value:
                        values.append(build_field_paths(value)[0])
                    field_paths.append(other(field.path, values))
                elif isinstance(field.value, Copybook):
                    field_paths += build_field_paths(field.value)
                else:
                    field_paths.append(other(field.path, field.value))
            return field_paths

        paths = build_field_paths(self)
        return DictTools.fold(paths)

    def build_model(self, model_class, save=False):
        "return a new instance of a model populated with data from this Copybook instance"
        model = CascadeModel.populate(model_class, self)
        if save:
            CascadeModel.save(model)
        return model

    @turn_off_flags
    def _build_field_list(self):
        "build a list of fields in this copybook"
        fields = []
        for field_name in dir(self):
            if not re.match(r'^__', field_name):
                field = getattr(self, field_name)
                if hasattr(field, 'order'):
                    fields.append(field)
        return sorted(fields, key=lambda field: field.order)

    @classmethod
    def from_dict(cls, dictionary):
        """
        this will create an instance of a copybook object from a given dictionary
        """
        copybook = cls(to_copybook=True)
        copybook.populate_from_dict(dictionary)
        return copybook

    @classmethod
    def from_model(cls, instance):
        """
        this will populate a copybook object with data from a given model
        """
        copybook = cls(to_copybook=True)
        copybook.populate_from_model(instance)
        return copybook

    @classmethod
    def from_model_list(cls, instance_list):
        """
        this will populate a copybook object with data from the passed in list of models
        this will work like 'first come first serve', so if the same field is defined in multiple
        models, the model first in the list will win and set the value
        """
        copybook = cls(to_copybook=True)
        copybook.populate_from_model_list(instance_list)
        return copybook

    @classmethod
    def from_model_dict(cls, instance_dict):
        """
        this will populate a copybook object with data from a dictionary of models
        if this is used, then the field mappings on the coypbook's fields should specify which model
        to retrieve the fields from by adding a node to the beginning of the mapping that matches
        the model's key passed into this function
        """
        copybook = cls(to_copybook=True)
        copybook.populate_from_model_dict(instance_dict)
        return copybook

    @classmethod
    def from_record(cls, record):
        """
        this will populate a copybook object with data from a passed in, single line record string
        """
        copybook = cls(to_model=True)
        copybook.populate_from_record(record)
        return copybook

class PopulateModel(object):

    def __init__(self, copybook):
        self.copybook = copybook

    def __call__(self, model):
        model_fields = dict([(field.name, field) for field in model._meta.fields])
        data = self.copybook.to_dict() if isinstance(self.copybook, Copybook) else self.copybook
        for name, value in data.items():
            self.set_field(model, model_fields, name, value)
        return model

    def populate_related_models(self, model, name, value):
        "in order for related models to work, the current model and each related model must be saved"
        model.save()
        related_manager = getattr(model, name)
        for item in value:
            new_model = CascadeModel.populate(related_manager.create(), item)
            new_model.save()
        return model

    def set_field(self, model, model_fields, name, value):
        if name in model_fields:
            field = model_fields[name]
            if isinstance(field, ForeignKey):
                try:
                    CascadeModel.populate(getattr(model, name), value)
                except field.rel.to.DoesNotExist:
                    CascadeModel.populate(field.rel.to, value)
            else:
                setattr(model, name, value)
        elif name in dir(model) and type(value) == list:
            self.populate_related_models(model, name, value)

class CascadeModel(object):
    """
    This class can be used to traverse a django model's relationships and do 'something' to an entire django
    model structure and all related models
    """

    @staticmethod
    def save(model):
        "save a model and all related models"
        return CascadeModel(lambda model_inst: model_inst.save())._cascade(model)

    @staticmethod
    def populate(main_model, copybook):
        "populate a model and related models given a copybook"
        main_model = main_model if type(main_model).__class__.__name__ != 'type' else main_model()
        return CascadeModel(PopulateModel(copybook))._cascade(main_model)

    def __init__(self, model_operation=lambda model:None, field_operation=lambda model, field_name:None):
        "operation is something that needs to be done to a model"
        self.model_operation = model_operation
        self.field_operation = field_operation

    def _get_foreign_keys(self, model, model_fields):
        "return a list of foreign keys for a given model"
        foreign_keys = []
        for name, field in model_fields:
            if isinstance(field, ForeignKey):
                try:
                    foreign_keys.append((name, getattr(model, name)))
                except field.rel.to.DoesNotExist:
                    foreign_keys.append((name, field.rel.to()))
            else:
                self.field_operation(model, name)
        return foreign_keys

    def _cascade_nested_foreign_keys(self, foreign_keys):
        "cascade all foreign keys and follow their relationships as well"
        for _, model in foreign_keys:
            self._cascade(model)

    def _operate_on_related_models(self, model, related_models):
        "perofrm the given operation on each foreign key object"
        for name, foreign_key in related_models:
            self.model_operation(foreign_key)
            setattr(model, name, foreign_key) #this is a hack, needed by Django for some dumb reason

    def _cascade(self, model):
        "this will perform a given operation on a model as well as all related models"
        instance_fields = [(field.name, field) for field in model._meta.fields]

        foreign_keys = self._get_foreign_keys(model, instance_fields)
        if foreign_keys:
            self._cascade_nested_foreign_keys(foreign_keys)
            self._operate_on_related_models(model, foreign_keys)

        self.model_operation(model)
        return model



from django import db

__all__ = ('First', 'Second', 'Third', 'Fourth', 'Fifth',
           'Sixth', 'Seventh', 'Eights', 'Ninth', 'Tenth', 'Eleventh', 'Twelfth')

class First(db.models.Model):
    first_field = db.models.CharField(max_length=10)
    second_field = db.models.CharField(max_length=10)

class Second(db.models.Model):
    third_field = db.models.CharField(max_length=10)

class Third(db.models.Model):
    second = db.models.ForeignKey(Second)

class Fourth(db.models.Model):
    third = db.models.ForeignKey(Third)

class Fifth(db.models.Model):
    fourth = db.models.ForeignKey(Fourth)

class Sixth(db.models.Model):
    fifth = db.models.ForeignKey(Fifth)

class Seventh(db.models.Model):
    fourth_field = db.models.CharField(max_length=10)

class Eights(db.models.Model):
    sevenths = db.models.ManyToManyField(Seventh)

class Ninth(db.models.Model):
    first = db.models.ForeignKey(First)

class Tenth(db.models.Model):
    ninths = db.models.ManyToManyField(Ninth)

class Eleventh(First):
    third_field = db.models.CharField(max_length=10)

class Twelfth(db.models.Model):
    fourth_field = db.models.CharField(max_length=10)
    fifth_field = db.models.ForeignKey(First)



from djcopybook import models, fields

__all__ = (
    "TestCopybook", "FirstCopybook", "SecondCopybook",
    "ThirdCopybook", "FourthCopybook", "FifthCopybook",
    "SixthCopybook"
)

class TestCopybook(models.Copybook):
    first_field = fields.StringField(length=2, order=5)
    second_field = fields.IntegerField(length=2, order=4)
    third_field = fields.StringField(length=3, order=3)
    fourth_field = fields.IntegerField(length=4, order=2)
    fifth_field = fields.StringField(length=10, order=1)

class FirstCopybook(models.Copybook):
    first_name = fields.StringField(length=40, order=1, path="first_field")
    last_name = fields.StringField(length=50, order=2, path="second_field")

class SecondCopybook(models.Copybook):
    one = fields.StringField(length=20, order=1, path="fourth_field")
    two = fields.StringField(length=20, order=2, path="fifth_field.first_field")
    three = fields.StringField(length=20, order=3, path="fifth_field.second_field")

class ThirdCopybook(models.Copybook):
    one = fields.StringField(length=20, order=1, path="first_field")
    two = fields.StringField(length=20, order=2, path="second_field")
    three = fields.StringField(length=20, order=3, path="fourth_field")

class FourthCopybook(models.Copybook):
    one = fields.StringField(length=20, order=1, path="one.first_field")
    two = fields.StringField(length=20, order=2, path="two.first_field")

class FifthCopybook(models.Copybook):
    one = fields.StringField(length=10, order=1, path="first_field")
    two = fields.StringField(length=10, order=2, path="second_field")

class SixthCopybook(models.Copybook):
    one = fields.StringField(length=2, order=1, path="first_field")
    two = fields.StringField(length=3, order=2, path="second_field")



import datetime

__all__ = (
    'StringField',
    'IntegerField',
    'DateField',
    'DecimalField',
    'FragmentField',
    'ListField',
)

class NOT_PROVIDED(object):
    pass


class FieldLengthError(Exception):
    pass


def str_padding(length, val):
    """Formats value giving it a right space padding up to a total length of 'length'"""
    return '{0:<{fill}}'.format(val, fill=length)


def int_padding(length, val):
    """Formats value giving it left zeros padding up to a total length of 'length'"""
    return '{0:0>{fill}}'.format(val, fill=length)


def float_padding(length, val, decimals=2):
    """Pads zeros to left and right to assure proper length and precision"""
    return '{0:0>{fill}.{precision}f}'.format(float(val), fill=length, precision=decimals)


def is_blank_string(val):
    return isinstance(val, basestring) and val.strip() == ''


class FixedWidthField(object):
    # set a default validators?
    attname = ''       # will get set to the name of the field this descriptor is being used as
    auto_truncate = '' # Gets set locally when record is initialized.

    # Tracks each time a Field instance is created. Used to retain order.
    creation_counter = 0

    def __init__(self, length, default=NOT_PROVIDED):
        self.length = length
        self.default = default

        # Increase the creation counter, and save our local copy.
        self.creation_counter = FixedWidthField.creation_counter
        FixedWidthField.creation_counter += 1

    def __get__(self, instance, txpe):
        try:
            return getattr(instance, self._get_instance_field())
        except AttributeError:
            return self.get_default()

    def __set__(self, instance, val):
        setattr(instance, self._get_instance_field(), self.to_python(val))

    def _get_instance_field(self):
        return "{attname}_{creation_counter}".format(**self.__dict__)

    def has_default(self):
        return self.default is not NOT_PROVIDED

    def get_default(self):
        if self.has_default():
            if callable(self.default):
                return self.default()
            return self.default

    def to_python(self, val):
        if val is None:
            return val
        return str(val).rstrip()

    def to_record(self, val):
        if val is None:
            val = ''
        return str_padding(self.length, val)

    def get_record_value(self, val):
        record_val = self.to_record(val)
        if self.auto_truncate:
            record_val = record_val[:self.length]
        self._check_record_length(record_val)
        return record_val

    def _check_record_length(self, record_val):
        if len(record_val) > self.length:
            err = "'{attname}' value '{0}' is longer than {length} chars.".format(record_val, **self.__dict__)
            raise FieldLengthError(err)


class StringField(FixedWidthField):
    pass


class IntegerField(FixedWidthField):

    def to_python(self, val):
        if val is None or is_blank_string(val):
            return None
        return int(val)

    def to_record(self, val):
        if val is None:
            val = 0
        return int_padding(self.length, val)


class DecimalField(FixedWidthField):

    def __init__(self, length, default=NOT_PROVIDED, decimals=2):
        self.decimals = decimals
        super(DecimalField, self).__init__(length, default)

    def to_python(self, val):
        if val is None or is_blank_string(val):
            return None
        return float(val)

    def to_record(self, val):
        if val is None:
            val = 0
        return float_padding(self.length, val, decimals=self.decimals)


class DateField(FixedWidthField):

    def __init__(self, length, default=NOT_PROVIDED, format="%Y-%m-%d"):
        self.format = format
        super(DateField, self).__init__(length, default)

    def to_python(self, val):
        if val is None or is_blank_string(val):
            return None
        if isinstance(val, datetime.datetime):
            return val.date()
        if isinstance(val, datetime.date):
            return val

        return datetime.datetime.strptime(val, self.format).date()

    def to_record(self, val):
        if not val:
            return str_padding(self.length, '')
        return val.strftime(self.format)


class FragmentField(FixedWidthField):
    """
    Allows you to create a field on a record that is itself a complete
    record. Similar to a ``ListField`` except it only occurs once.


    class Phone(Record):
      area_code = fields.IntegerField(length=3)
      prefix = fields.IntegerField(length=3)
      line_number = fields.IntegerField(length=4)

    class Contact(Record):
      name = fields.StringField(length=100)
      phone_number = fields.FragmentField(record=Phone)
      email = fields.StringField(length=100)

    """

    def __init__(self, record):
        self.record_class = record
        super(FragmentField, self).__init__(len(record))

    def to_python(self, val):
        """
        :returns:
            Always returns an instance of the record class.
        """
        if val is None:
            return self.record_class()
        if isinstance(val, basestring):
            return self.record_class.from_record(val)
        elif isinstance(val, self.record_class):
            return val
        else:
            msg = "Redefined field must be a string or {record} instance.".format(
                record=self.record_class.__name__)
            raise TypeError(msg)

    def to_record(self, val):
        """
        :param val:
            val will either be None or an instance of ``self.record_class``
        :returns:
            Always returns a string spaced properly for self.record_class.
        """
        if val is None:
            return self.record_class().to_record()
        return val.to_record()


class ListField(FixedWidthField):
    """
    ListField allows you to have a field made up of a number of
    other records. Similar to COBOL's OCCURS.

    parameters:
      - record: which Record the field is made up of
      - length: how many times that record occurs

    """

    def __init__(self, record, length=1):
        self.record_class = record
        super(ListField, self).__init__(length)

    def _get_records_from_string(self, val):
        records = []
        record_len = len(self.record_class)
        for _ in range(self.length):
            records.append(self.record_class.from_record(val[:record_len]))
            val = val[record_len:]
        return records

    def to_python(self, val):
        """
        the python representation should be a list of instantiated
        Record classes.
        """
        if isinstance(val, basestring):
            return self._get_records_from_string(val)
        elif not all([isinstance(r, self.record_class) for r in val]):
            msg = "List field must contain instances of '{0}'.".format(self.record_class.__name__)
            raise TypeError(msg)
        return list(val)

    def get_default(self):
        return []

    def to_record(self, val):
        """
        We receive a list of Record classes and must make sure
        we have a complete record we're giving back.
        """
        while len(val) < self.length:
            val.append(self.record_class())
        return ''.join([v.to_record() for v in val])

    def _check_record_length(self, record_val):
        max_record_length = len(self.record_class)
        record_length = len(record_val)
        if record_length > (self.length * max_record_length):
            record_count = record_length / max_record_length
            msg = "'{attname}' contains {cnt} records but can only have {length}.".format(cnt=record_count,
                                                                                          **self.__dict__)
            raise FieldLengthError(msg)




"""
Borrowed extensively from django's forms for the setup and base
functionality of the 'Record' class. Django's forms do a nice
job of conveniently remembering the order of fields which is
a necessity for a properly formatted fixed-width record.
"""

# using django's SortedDict to preserve python 2.6 compatibility
from django.utils.datastructures import SortedDict as OrderedDict

from copy import deepcopy

from djcopybook.fixedwidth import fields

__all__ = ('Record',)

def get_declared_fields(bases, attrs):
    """
    Create a list of fixedwidth field instances from the passed in 'attrs', plus any
    similar fields on the base classes (in 'bases').
    """
    fw_fields = [(field_name, obj) for field_name, obj in attrs.items() if isinstance(obj, fields.FixedWidthField)]
    fw_fields.sort(key=lambda x: x[1].creation_counter)

    # If this class is subclassing another Record, add that Record's fields.
    # Note that we loop over the bases in *reverse*. This is necessary in
    # order to preserve the correct order of fields.
    for base in bases[::-1]:
        if hasattr(base, 'base_fields'):
            fw_fields = base.base_fields.items() + fw_fields
    return OrderedDict(fw_fields)


class DeclarativeFieldsMetaclass(type):
    """
    Metaclass that converts Field attributes to a dictionary called
    'base_fields', taking into account parent class 'base_fields' as well.
    """
    def __new__(cls, name, bases, attrs):
        attrs['base_fields'] = get_declared_fields(bases, attrs)
        new_class = super(DeclarativeFieldsMetaclass,
                     cls).__new__(cls, name, bases, attrs)

        # useful to let each FixedWidthField field know its attribute name
        for field_name, field in new_class.base_fields.items():
            setattr(field, 'attname', field_name)

        return new_class

    def __len__(cls):
        """
        Total length this record will be in a fixed width format.
        """
        return sum(get_field_length(f) for f in cls.base_fields.values())


class BaseRecord(object):

    # This is the main implementation of all the record logic. Note that this
    # class is different than Record. See the comments by the Record class for more
    # information. Any improvements to the fixedwidth API should be made to *this*
    # class, not to the Record class.

    auto_truncate = False

    def __init__(self, **kwargs):
        # The base_fields class attribute is the *class-wide* definition of
        # fields. Because a particular *instance* of the class might want to
        # alter self.fields, we create self.fields here by copying base_fields.
        # Instances should always modify self.fields; they should not modify
        # self.base_fields.
        self.fields = deepcopy(self.base_fields)

        # populate the fixedwidth fields from the keyword arguments passed in.
        fields_iter = iter(self.fields.values())
        for field in fields_iter:
            if kwargs:
                try:
                    val = kwargs.pop(field.attname)
                except KeyError:
                    # This is done with an exception rather than the
                    # default argument on pop because we don't want
                    # get_default() to be evaluated, and then not used.
                    val = field.get_default()
            else:
                val = field.get_default()

            field.auto_truncate = self.auto_truncate
            setattr(self, field.attname, val)

    def __len__(self):
        """
        Total length this record will be in a fixed width format.
        """
        return len(self.__class__)

    def __str__(self):
        return self.to_record()

    def get_record_value(self, fieldname):
        """
        Allows you to obtain the fixedwidth value for a particular fieldname
        """
        field_class = self.fields[fieldname]
        return field_class.get_record_value(getattr(self, fieldname))

    def to_record(self):
        """
        Strings together all fields as one combined record value.
        """
        return ''.join(self.get_record_value(fn) for fn in self.fields)

    @classmethod
    def from_record(cls, record):
        """
        Takes an existing fixed width record and breaks it into it's
        python Record object.
        """
        if len(record) != len(cls):
            raise ValueError("Fixed width record length is {} but should be {}.".format(len(record), len(cls)))

        new_record = cls()

        pos = 0
        for attrname, field_class in cls.base_fields.items():
            field_length = get_field_length(field_class)
            field_value = field_class.to_python(record[pos:pos + field_length])
            setattr(new_record, attrname, field_value)

            pos += field_length
        return new_record


class Record(BaseRecord):
    """A collection of FixedWidthFields, plus their associated data."""
    # This is a separate class from BaseRecord in order to abstract the way
    # self.fields is specified. This class (Record) is the one that does the
    # fancy metaclass stuff purely for the semantic sugar -- it allows one
    # to define a fixedwidth using declarative syntax.
    # BaseCopybook itself has no way of designating self.fields.
    __metaclass__ = DeclarativeFieldsMetaclass


def get_field_length(f):
    """
    Normally field length is the length attribute of a FixedWidthField
    class. However, on ListField classes the length attribute represents
    how many times the record is repeated, so we need the total.
    """
    if isinstance(f, fields.ListField):
        return f.length * len(f.record_class)
    return f.length




import datetime
from django import test
from django.utils.unittest import TestCase
from djcopybook import models, fields
from sample import models as sample_models
from sample import copybooks

class DictToolsTests(test.TestCase):

    def test_be_able_to_merge_two_dicts_into_single_one(self):
        d1 = { 'first_field': {'second_field': {'third_field': 1, }}}
        d2 = { 'first_field': {'second_field': {'fourth_field':2, }, 'fifth_field':3, }, 'sixth_field':4,}

        self.assertEqual({'first_field':
                            {'second_field':
                                {'fourth_field':2,
                                 'third_field':1, },
                             'fifth_field':3, },
                          'sixth_field':4, },
                         models.DictTools.merge(d1, d2))

    def test_be_able_to_fold_a_list_of_dicts_into_a_single_one(self):
        d1 = {'first_field':{'second_field':{'third_field':1, }}}
        d2 = {'first_field':{'second_field':{'fourth_field':2, }, 'fifth_field':3, }, 'sixth_field':4, }
        d3 = {'second_field':{'third_field':84848}}
        d4 = {'second_field':{'first_field':{'third_field':'XXXX'}}}
        d5 = {'first_field':{'second_field':{'tenth_field':200}}}

        self.assertEqual({'first_field':
                            {'second_field':
                                {'fourth_field':2,
                                 'third_field':1,
                                 'tenth_field':200, },
                             'fifth_field':3, },
                          'sixth_field':4,
                          'second_field':{
                              'third_field':84848,
                              'first_field':{'third_field':'XXXX'},
                          }},
                         models.DictTools.fold((d1, d2, d3, d4, d5)))

    def test_handle_lists(self):
        d1 = { 'first_field': {'second_field': {'third_field': 1, 'x':['1', '2', '3', '4', '5']}}}
        d2 = { 'first_field': {'second_field': {'fourth_field':2, }, 'fifth_field':3, }, 'sixth_field':4, }

        self.assertEqual({'first_field':
                            {'second_field':
                                {'fourth_field':2,
                                 'third_field':1,
                                 'x':['1', '2', '3', '4', '5', ], },
                             'fifth_field':3, },
                          'sixth_field':4, },
                         models.DictTools.merge(d1, d2))

    def test_handle_dicts_nested_in_lists(self):
        d1 = { 'first_field': {'second_field': {'third_field': 1, 'x':[
                {'one':1, 'two':2}, {'one':'I', 'two':'II'}, {'one':'One', 'two':'Two'}, ]}}}
        d2 = { 'first_field': {'second_field': {'fourth_field':2, }, 'fifth_field':3, }, 'sixth_field':4, }

        self.assertEqual({'first_field':
                            {'second_field':
                                {'fourth_field':2,
                                 'third_field':1,
                                 'x':[{'one':1, 'two':2}, {'one':'I', 'two':'II'}, {'one':'One', 'two':'Two'}, ], },
                             'fifth_field':3, },
                          'sixth_field':4, },
                         models.DictTools.merge(d1, d2))

class ModelToDictTests(test.TestCase):
    def test_convert_model_into_dict(self):
        model_dict = models.model_to_dict(sample_models.First(first_field='asdf', second_field="xxx"))
        self.assertEqual(model_dict, {'first_field':'asdf', 'id':None, 'second_field':'xxx'})

    def test_include_inherited_model_fields(self):
        eleventh = sample_models.Eleventh(first_field='123', second_field='456', third_field='789')

        self.assertEqual(models.model_to_dict(eleventh),
                         {'id':None,
                          'first_field':'123',
                          'second_field':'456',
                          'third_field':'789'})

class RedefinedFieldTests(test.TestCase):

    def test_resolve_to_a_nested_copybook(self):
        class Dummy(models.Copybook):
            field = fields.RedefinedField(length=60, order=1, select_func='select_func')

            def select_func(self, record):
                return copybooks.SecondCopybook

        dummy = Dummy.from_record("x" * 20 + "y" * 20 + "z" * 20)
        dummy.to_model = False
        self.assertEqual(type(dummy.field), fields.NestedCopybook)

    def test_call_select_method_to_determine_which_copybook_to_use(self):
        class Dummy(models.Copybook):
            field = fields.RedefinedField(length=60, order=1, select_func='select_copybook_for_field')

            def select_copybook_for_field(self, redefined_record):
                return copybooks.SecondCopybook

        data = "x" * 20 + "y" * 20 + "z" * 20
        dummy = Dummy.from_record(data)
        dummy.to_model = False
        self.assertEqual(type(dummy.field.value), copybooks.SecondCopybook)
        self.assertEqual(dummy.field.value.one, "x" * 20)
        self.assertEqual(dummy.field.value.two, "y" * 20)
        self.assertEqual(dummy.field.value.three, "z" * 20)
        dummy.to_copybook = True
        self.assertEqual(str(dummy.field), data)

    def test_create_dict_according_to_logic_for_redefined_copybook(self):
        class Dummy(models.Copybook):
            x = fields.StringField(length=10, order=2, path="something")
            field = fields.RedefinedField(length=60, order=1, select_func='select_copybook_for_field')

            def select_copybook_for_field(self, redefined_record):
                return copybooks.SecondCopybook

        data = "x" * 20 + "y" * 20 + "z" * 20 + '1' * 10
        dummy = Dummy.from_record(data)
        self.assertEqual({'something':'1111111111',
                         'fourth_field': 'xxxxxxxxxxxxxxxxxxxx',
                          'fifth_field': {'first_field': 'yyyyyyyyyyyyyyyyyyyy',
                                          'second_field': 'zzzzzzzzzzzzzzzzzzzz'}}, dummy.to_dict())

class ListFieldTests(test.TestCase):
    def test_accept_copybook_and_occurances(self):
        fields.ListField(length=2, copybook=copybooks.FirstCopybook)

    def test_allow_values_to_be_set_as_list_and_returned_as_string(self):
        class DummyCopybookOne(models.Copybook):
            string_field = fields.StringField(length=1, order=1)
            int_field = fields.IntegerField(length=3, order=2)

        class DummyCopybookTwo(models.Copybook):
            list_field = fields.ListField(length=5, copybook=DummyCopybookOne)

        one = DummyCopybookOne(to_copybook=True)
        one.string_field = 'X'
        one.int_field = 123
        two = DummyCopybookOne(to_copybook=True)
        two.string_field = 'Y'
        two.int_field = 999
        three = DummyCopybookOne(to_copybook=True)
        three.string_field = 'Z'
        three.int_field = 4

        other = DummyCopybookTwo(to_copybook=True)
        other.list_field = [one, two, three]

        self.assertEqual(other.list_field, 'X123Y999Z004 000 000')

    def test_populate_list_field_given_model(self):
        one = sample_models.First(first_field="asdf", second_field="xxxx")
        two = sample_models.First(first_field="1111", second_field="2222")
        three = sample_models.First(first_field="aaaaa", second_field="hhhhh")

        class NineCopybook(models.Copybook):
            char_one = fields.StringField(length=3, path="first_field", order=1)
            char_two = fields.StringField(length=3, path="second_field", order=2)

        class Dummy(models.Copybook):
            nines = fields.ListField(length=5, copybook=NineCopybook, order=1)

        dummy = Dummy(to_copybook=True)
        dummy.nines = (NineCopybook.from_model(one), NineCopybook.from_model(two), NineCopybook.from_model(three))
        self.assertEqual(dummy.to_record(), "asdxxx111222aaahhh            ")

    def test_populate_list_field_given_dict(self):

        class NineCopybook(models.Copybook):
            char_one = fields.StringField(length=3, path="first_field", order=1)
            char_two = fields.StringField(length=3, path="second_field", order=2)

        class Dummy(models.Copybook):
            nines = fields.ListField(length=5, copybook=NineCopybook, order=1)

        dummy = Dummy(to_copybook=True)
        dummy.nines = [NineCopybook.from_dict(dict(first_field=str(i))) for i in range(5)]
        self.assertEqual(dummy.to_record(), "0     1     2     3     4     ")

    def test_be_able_to_populate_a_list_of_models_from_string(self):
        class Dummy(models.Copybook):
            field = fields.ListField(length=5, copybook=copybooks.SixthCopybook, order=1)

        dummy = Dummy(to_model=True)
        dummy.to_model = False
        dummy.field.strip_self_from_record("112223344455666")
        self.assertEqual(dummy.field.value[0].one, "11")
        self.assertEqual(dummy.field.value[0].two, "222")
        self.assertEqual(dummy.field.value[1].one, "33")
        self.assertEqual(dummy.field.value[1].two, "444")
        self.assertEqual(dummy.field.value[2].one, "55")
        self.assertEqual(dummy.field.value[2].two, "666")

    def test_call_fields_strip_from_record_when_copybook_is_reading_record(self):

        class MockList(fields.List):
            counter = 0
            def strip_self_from_record(self, record):
                self.counter += 1

        class MockListField(fields.ListField):
            default_class = MockList

        class DummyCopybook(models.Copybook):
            field = MockListField(length=5, copybook=copybooks.SixthCopybook, order=1)

        dummy = DummyCopybook(to_model=True)
        dummy.to_model = False
        dummy.populate_from_record("xxzzzaabbb")

        self.assertEqual(dummy.field.counter, 1)

    def test_be_able_to_turn_occuring_record_into_model(self):

        class NineCopybook(models.Copybook):
            char_one = fields.StringField(length=3, path="fourth_field", order=1)

        class Dummy(models.Copybook):
            nines = fields.ListField(length=5, copybook=NineCopybook, path="twelfth_set", order=1)

        copybook = Dummy.from_record("asdxxx111222aaahhh")
        model = copybook.build_model(sample_models.First)
        self.assertEqual(model.twelfth_set.all()[0].fourth_field, 'asd')
        self.assertEqual(model.twelfth_set.all()[1].fourth_field, 'xxx')
        self.assertEqual(model.twelfth_set.all()[2].fourth_field, '111')
        self.assertEqual(model.twelfth_set.all()[3].fourth_field, '222')
        self.assertEqual(model.twelfth_set.all()[4].fourth_field, 'aaa')

    def test_populate_related_models(self):
        model = models.PopulateModel(None).populate_related_models(sample_models.First(first_field='abc', second_field='xyz'),
            'twelfth_set', [
                {'fourth_field':'xxx'},
                {'fourth_field':'yyy'},
                {'fourth_field':'zzz'},
            ]
        )
        self.assertEqual(model.twelfth_set.count(), 3)

class StringCopybookFieldTests(test.TestCase):
    def test_accept_length_argument(self):
        string_field = fields.StringField(length=10)
        self.assertEqual(string_field.length, 10)

    def test_truncate_data_overflow(self):
        class Dummy(models.Copybook):
            string = fields.StringField(length=1)

        dummy = Dummy.from_model(sample_models.First)
        dummy.string = "ABC"
        self.assertEqual(str(dummy.string), "A")

    def test_pad_with_spaces_to_the_right(self):

        class Dummy(models.Copybook):
            string = fields.StringField(length=10)

        dummy = Dummy.from_model(sample_models.First)
        dummy.string = "X"
        self.assertEqual(str(dummy.string), "X         ")

    def test_be_able_to_set_default_value(self):
        class Dummy(models.Copybook):
            string = fields.StringField(length=10, default='xxx')

        dummy = Dummy.from_model(sample_models.First)
        self.assertEqual(str(dummy.string), "xxx       ")

class SignedIntegerCopybookFieldTests(test.TestCase):

    def setUp(self):
        class Dummy(models.Copybook):
            field = fields.SignedIntegerField(length=10)
        self.copybook_class = Dummy

    def test_add_plus_sign_to_positive_number(self):
        dummy = self.copybook_class(to_copybook=True)
        dummy.field = 123
        self.assertEqual("+000000123", str(dummy.field))

    def test_add_minus_sign_to_negative_number(self):
        dummy = self.copybook_class(to_copybook=True)
        dummy.field = -123
        self.assertEqual("-000000123", str(dummy.field))

    def test_stay_within_size_when_larger_negative_number_is_given(self):
        dummy = self.copybook_class(to_copybook=True)
        dummy.field = -123456789
        self.assertEqual("-123456789", str(dummy.field))

    def test_add_sign_to_zero(self):
        dummy = self.copybook_class(to_copybook=True)
        dummy.field = 0
        self.assertEqual("+000000000", str(dummy.field))

    def test_limit_string_size(self):
        dummy = self.copybook_class(to_copybook=True)
        dummy.field = 99999999999
        self.assertEqual("+999999999", str(dummy.field))

class SignedDecimalCopybookFieldTests(TestCase):

    def setUp(self):
        class SignedDecimalDummy(models.Copybook):
            field = fields.SignedDecimalField(length=9)
        self.copybook = SignedDecimalDummy(to_copybook=True)
        self.copybook.field = float('99999.99')

    def test_return_float_as_string_with_decimal(self):
        self.assertEqual("+99999.99", str(self.copybook.field))

    def test_add_minus_sign_if_negative(self):
        self.copybook.field = float('-99999.99')
        self.assertEqual("-99999.99", str(self.copybook.field))

    def test_add_sign_to_zero(self):
        self.copybook.field = 0
        self.assertEqual("+00000.00", str(self.copybook.field))

    def test_limit_size_of_field(self):
        self.copybook.field = float('9999999999.99')
        self.assertEqual("+99999.99", str(self.copybook.field))

    def test_pad_with_zeros(self):
        self.copybook.field = float('50.00')
        self.assertEqual("+00050.00", str(self.copybook.field))

    def test_be_empty_string_if_value_is_none(self):
        self.copybook.field = None
        self.assertEqual(' ' * 9, str(self.copybook.field))

    def test_add_cents_for_non_floats(self):
        self.copybook.field = 123
        self.assertEqual('+00123.00', str(self.copybook.field))

    def test_add_decimals_and_pad(self):
        self.copybook.field = 110.10
        self.assertEqual('+00110.10', str(self.copybook.field))

class IntegerCopybookFieldTests(test.TestCase):
    def test_accept_length_argument(self):
        integer_field = fields.IntegerField(length=10)
        self.assertEqual(integer_field.length, 10)

    def test_truncate_overflow(self):
        class Dummy(models.Copybook):
            integer = fields.IntegerField(length=1)

        dummy = Dummy.from_model(sample_models.First)
        dummy.integer = 12345
        self.assertEqual(str(dummy.integer), '1')

    def test_pad_with_zeros_to_the_left(self):
        class Dummy(models.Copybook):
            integer = fields.IntegerField(length=10)

        dummy = Dummy.from_model(sample_models.First)
        dummy.integer = 1
        self.assertEqual(str(dummy.integer), '0000000001')

    def test_be_able_to_set_default_value(self):
        class Dummy(models.Copybook):
            integer = fields.IntegerField(length=10, default=9)

        dummy = Dummy.from_model(sample_models.First)
        self.assertEqual(str(dummy.integer), '0000000009')

    def test_default_to_zero_if_receive_spaces(self):
        class Dummy(models.Copybook):
            integer = fields.IntegerField(length=10)

        dummy = Dummy.from_record('          ')
        self.assertEqual(dummy.integer, 0)


class DateCopybookFieldTests(test.TestCase):

    def test_accept_date_format(self):
        fields.DateField(length=6, format="%m%d%y")

    def test_convert_datestring_into_dateobject(self):
        date = fields.Date(value='101010', length=6, order=1, path='asdf', format="%m%d%y")
        self.assertEqual(datetime.datetime(2010, 10, 10, 0, 0), date.to_model())

    def test_convert_datetime_into_datestring_for_copybook(self):
        date = fields.Date(value=datetime.datetime(2010, 10, 10, 0, 0), length=6, order=1, path='asdf', format="%m%d%y")
        self.assertEqual('101010', date.to_copybook())

    def test_convert_date_into_datestring_for_copybook(self):
        date = fields.Date(value=datetime.date(2010, 10, 10), length=6, order=1, path='asdf', format="%m%d%y")
        self.assertEqual('101010', date.to_copybook())

    def test_return_string_of_spaces_when_no_value_in_to_copybook(self):
        date = fields.Date(value=None, length=6, order=1, path='asdf', format="%m%d%y")
        self.assertEqual('      ', date.to_copybook())

class ModelToCopybookTests(test.TestCase):
    def setUp(self):
        self.instance = sample_models.First(first_field="asdf", second_field="xyz")

    def test_force_instance_to_be_created_for_either_copybook_or_model(self):
        self.assertRaises(ValueError, models.Copybook)
        self.assertTrue(models.Copybook(to_copybook=True))
        self.assertTrue(models.Copybook(to_model=True))

class PopulateModelTests(test.TestCase):

    def test_populate_related_models_with_data(self):
        model = models.CascadeModel.populate(sample_models.Twelfth, {
                'fourth_field':'asdf',
                'fifth_field':{'first_field':'xxx999',
                'second_field':'zzz123', }
            },
        )

        self.assertEqual(model.fourth_field, 'asdf')
        self.assertEqual(model.fifth_field.first_field, 'xxx999')
        self.assertEqual(model.fifth_field.second_field, 'zzz123')

    def test_save_all_related_models(self):
        model = models.CascadeModel.populate(sample_models.Twelfth, {
            'fourth_field':'asdf',
            'fifth_field': {'first_field':'xxx999', 'second_field':'zzz123'},
            }
        )

        models.CascadeModel.save(model)

        self.assertEqual(model.fifth_field.pk, 1)
        self.assertEqual(model.pk, 1)

    def test_populate_indefinately_deeply_nested_models_with_data(self):
        model = models.CascadeModel.populate(sample_models.Sixth,
            {'fifth':{
                'fourth':{
                    'third':{
                        'second':{
                            'third_field':'asdf'
                        }
                    }
                }
            }},
        )

        self.assertEqual(model.fifth.fourth.third.second.third_field, 'asdf')

    def test_save_indefinately_deeplly_nested_model_instances(self):
        model = models.CascadeModel.populate(sample_models.Sixth,
            {'fifth':{
                'fourth':{
                    'third':{
                        'second':{
                            'third_field':'asdf'
                        }
                    }
                }
            }},
        )

        models.CascadeModel.save(model)

        self.assertEqual(model.fifth.fourth.third.second.pk, 1)
        self.assertEqual(model.fifth.fourth.third.pk, 1)
        self.assertEqual(model.fifth.fourth.pk, 1)
        self.assertEqual(model.fifth.pk, 1)
        self.assertEqual(model.pk, 1)

class CopybookTests(test.TestCase):

    def setUp(self):
        self.copybook = copybooks.TestCopybook.from_model(sample_models.First())
        self.copybook.first_field = "X"
        self.copybook.second_field = 1
        self.copybook.third_field = "V"
        self.copybook.fourth_field = 3
        self.copybook.fifth_field = "D"

    def test_convert_copybook_into_dict(self):
        copybook = copybooks.SecondCopybook(to_copybook=True)
        copybook.one = "1"
        copybook.two = "2"
        copybook.three = "3"

        self.assertEqual(copybook.to_dict(), {
            'fourth_field':'1',
            'fifth_field': {
                'first_field':'2',
                'second_field':'3',
            },
        })

    def test_be_able_to_get_fields_back_in_order(self):
        self.copybook.to_model = False
        self.copybook.to_copybook = False
        copybook_fields = [self.copybook.fifth_field, self.copybook.fourth_field,
                  self.copybook.third_field, self.copybook.second_field, self.copybook.first_field]

        self.assertEqual(self.copybook.get_fields(), copybook_fields)

    def test_turn_all_fields_into_single_string(self):
        self.assertEqual(self.copybook.to_record(), "D         0003V  01X ")

    def test_turn_a_single_string_record_into_a_copybook_object(self):
        new_copybook = copybooks.TestCopybook.from_record("D         0003V  01X ")

        self.assertEqual(new_copybook.first_field, "X ")
        self.assertEqual(new_copybook.second_field, 1)
        self.assertEqual(new_copybook.third_field, "V  ")
        self.assertEqual(new_copybook.fourth_field, 3)
        self.assertEqual(new_copybook.fifth_field, "D         ")

    def test_populate_copybook_given_a_model(self):
        model = sample_models.First(first_field="asdf", second_field="xyz")
        self.assertEqual(model.first_field, "asdf")
        self.assertEqual(model.second_field, "xyz")

        new_copybook = copybooks.FirstCopybook.from_model(model)
        new_copybook.to_model = False
        new_copybook.to_copybook = False
        self.assertEqual(new_copybook.first_name.value, "asdf")
        self.assertEqual(new_copybook.last_name.value, "xyz")

    def test_allow_multiple_models_to_be_passed(self):
        first = sample_models.First(first_field="asdf", second_field="xyz")
        twelfth = sample_models.Twelfth(fourth_field="XX", fifth_field=first)

        copybook = copybooks.ThirdCopybook.from_model_list((first, twelfth))
        copybook.to_model = False
        copybook.to_copybook = False
        self.assertEqual(copybook.one.value, "asdf")
        self.assertEqual(copybook.two.value, "xyz")
        self.assertEqual(copybook.three.value, "XX")

    def test_allow_multiple_models_to_be_passed_in_dict(self):
        first_one = sample_models.First(first_field="asdf", second_field="xyz")
        first_two = sample_models.First(first_field="1234", second_field="789")

        copybook = copybooks.FourthCopybook.from_model_dict({'one': first_one, 'two': first_two})
        copybook.to_model = False
        copybook.to_copybook = False
        self.assertEqual(copybook.one.value, "asdf")
        self.assertEqual(copybook.two.value, "1234")

    def test_return_a_model_instance_given_a_string(self):
        copybook = copybooks.FifthCopybook.from_record("XXXXXXXXXXAAAAAAAAAA")
        model = copybook.build_model(sample_models.First)
        self.assertEqual(model.first_field, "XXXXXXXXXX")
        self.assertEqual(model.second_field, "AAAAAAAAAA")
        self.assertEqual(model.pk, None)

    def test_return_a_saved_model_instance_given_a_string_and_save_flag(self):
        copybook = copybooks.FifthCopybook.from_record("XXXXXXXXXXAAAAAAAAAA")
        model = copybook.build_model(sample_models.First, save=True)
        self.assertEqual(model.first_field, "XXXXXXXXXX")
        self.assertEqual(model.second_field, "AAAAAAAAAA")
        self.assertEqual(model.pk, 1)

    def test_populate_multiple_models_if_requested(self):
        copybook = copybooks.ThirdCopybook.from_record("111111111111111111112222222222222222222233333333333333333333")
        first = copybook.build_model(sample_models.First)
        seventh = copybook.build_model(sample_models.Seventh)

        self.assertEqual(first.first_field, '11111111111111111111')
        self.assertEqual(first.second_field, '22222222222222222222')
        self.assertEqual(seventh.fourth_field, '33333333333333333333')

class DecimalFieldTests(test.TestCase):

    def test_return_positive_decimal_from_string(self):
        number = fields.Decimal(value="+0001544.00", length=11, order=1, path=None)
        self.assertEqual(number.to_model(), 1544.0)

    def test_return_negative_decimal_from_string(self):
        number = fields.Decimal(value="-9001544.87", length=11, order=1, path=None)
        self.assertEqual(number.to_model(), -9001544.87)

    def test_return_positive_integer_from_string(self):
        number = fields.Decimal(value="+9001544", length=11, order=1, path=None)
        self.assertEqual(number.to_model(), 9001544)

    def test_return_negative_integer_from_string(self):
        number = fields.Decimal(value="-9001544", length=11, order=1, path=None)
        self.assertEqual(number.to_model(), -9001544)

class NumberOrStringFieldTests(test.TestCase):

    def setUp(self):
        class Dummy(models.Copybook):
            field = fields.NumberOrStringField(length=10, order=1)
        self.copybook_class = Dummy

    def test_default_to_question_marks(self):
        dummy = self.copybook_class(to_copybook=True)
        self.assertEqual('??????????', dummy.field)



from setuptools import setup, find_packages

REQUIREMENTS = [
    'django',
]

from djcopybook import VERSION

setup(
    name="django-copybook",
    version=VERSION,
    author="Matthew J. Morrison",
    author_email="mattj.morrison@gmail.com",
    description="Convert Objects and Django models to/from fixed format records.",
    long_description=open('README.txt', 'r').read(),
    packages=find_packages(exclude=["example"],),
    install_requires=REQUIREMENTS,
    tests_require=REQUIREMENTS,
    test_suite='runtests.runtests',
    zip_safe=False,
    classifiers = [
        "Development Status :: 4 - Beta",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing",
        "Topic :: Utilities",
    ],
)




from datetime import date, datetime
from unittest import TestCase

from djcopybook import fixedwidth
from djcopybook.fixedwidth import fields

__all__ = (
    'PaddingTests',
    'RecordTests',
    'FixedWidthFieldTests',
    'StringFieldTests',
    'IntegerFieldTests',
    'DecimalFieldTests',
    'DateFieldTests',
    'ListFieldTests',
    'FragmentFieldTests',
)

class RecordOne(fixedwidth.Record):
    field_one = fields.StringField(length=5, default="AA")
    field_two = fields.IntegerField(length=7)

class RecordTwo(RecordOne):
    auto_truncate = True

    field_three = fields.DecimalField(length=9)
    field_four = fields.DateField(length=2)

class RecordThree(fixedwidth.Record):
    frag = fields.FragmentField(record=RecordOne)
    other_field = fields.StringField(length=3, default="BBB")


class PaddingTests(TestCase):

    def test_str_padding_pads_right_with_spaces(self):
        self.assertEqual("ALM   ", fields.str_padding(6, "ALM"))
        self.assertEqual("5     ", fields.str_padding(6, 5))

    def test_int_padding_pads_left_with_zeros(self):
        self.assertEqual("000010", fields.int_padding(6, 10))
        self.assertEqual("000010", fields.int_padding(6, "10"))

    def test_float_padding_pads_left_and_decimals_with_zeros(self):
        self.assertEqual("000010", fields.float_padding(6, 10, 0))
        self.assertEqual("000010", fields.float_padding(6, "10", 0))

        self.assertEqual("00300.12", fields.float_padding(8, 300.1234, 2))
        self.assertEqual("010.12", fields.float_padding(6, "10.1234", 2))
        self.assertEqual("010.10", fields.float_padding(6, "10.1", 2))
        self.assertEqual("010.00", fields.float_padding(6, 10, 2))

        self.assertEqual("000010.123", fields.float_padding(10, 10.1234, 3))
        self.assertEqual("0005.123", fields.float_padding(8, "5.1234", 3))


class RecordTests(TestCase):

    def test_keeps_list_of_fields_in_order_when_record_is_instantiated(self):
        r = RecordOne()
        self.assertEqual(['field_one', 'field_two'], r.fields.keys())
        self.assertEqual(['field_one', 'field_two'], r.base_fields.keys())

    def test_sets_auto_truncate_field_on_fields_when_instantiated(self):
        r = RecordOne() # defaults auto_truncate to False

        self.assertEqual(2, len(r.fields)) # sanity check
        for field in r.fields.values():
            self.assertEqual(False, field.auto_truncate)

    def test_sets_auto_truncate_field_on_fields_when_turned_on(self):
        r = RecordTwo() # this record has auto_truncate turned on

        self.assertEqual(4, len(r.fields)) # sanity check
        for field in r.fields.values():
            self.assertEqual(True, field.auto_truncate)

    def test_record_inheritance_maintains_order(self):
        r = RecordTwo()
        self.assertEqual(['field_one', 'field_two', 'field_three', 'field_four'], r.fields.keys())
        self.assertEqual(['field_one', 'field_two', 'field_three', 'field_four'], r.base_fields.keys())

    def test_fills_attributes_with_values_given_in_init(self):
        r = RecordOne(field_one=10, field_two=5)
        self.assertEqual("10", r.field_one)
        self.assertEqual(5, r.field_two)

    def test_to_record_returns_record_for_each_field(self):
        r = RecordOne(field_one="test", field_two=500)
        self.assertEqual("test 0000500", r.to_record())

    def test_returns_field_default_value_when_not_given(self):
        r = RecordOne(field_two=10)
        self.assertEqual("AA", r.field_one)

    def test_ignores_kwarg_value_when_field_not_declared_on_record(self):
        r = RecordOne(field_one="abc", no_field="test")
        self.assertEqual("abc", r.field_one)
        self.assertFalse(hasattr(r, "no_field"))

    def test_len_on_record_returns_total_fixed_width_length(self):
        r = RecordOne()
        self.assertEqual(12, len(r))

    def test_len_on_record_returns_total_amount_with_list_field(self):
        class ListRecord(fixedwidth.Record):
            field_one = fields.StringField(length=1)
            field_two = fields.IntegerField(length=2)

        class TestRecord(fixedwidth.Record):
            name = fields.StringField(length=2)
            list_field = fields.ListField(ListRecord, length=2)
            end = fields.StringField(length=1)

        # should get same len whether using instance or just the class
        r = TestRecord()
        self.assertEqual(9, len(r))
        self.assertEqual(9, len(TestRecord))

    def test_str_on_record_returns_to_record_value(self):
        r = RecordOne(field_one="test", field_two=500)
        self.assertEqual("test 0000500", str(r))

    def test_from_record_turns_fixed_width_string_to_record_object(self):
        r = RecordOne.from_record("test 0000500")
        self.assertEqual("test", r.field_one)
        self.assertEqual(500, r.field_two)

    def test_from_record_turns_fixed_width_string_to_record_with_list_field(self):
        class ListRecord(fixedwidth.Record):
            field_one = fields.StringField(length=1)
            field_two = fields.IntegerField(length=2)

        class TestRecord(fixedwidth.Record):
            name = fields.StringField(length=2)
            list_field = fields.ListField(ListRecord, length=2)
            end = fields.StringField(length=1)

        r = TestRecord.from_record("XXA05B10Z")
        self.assertEqual("XX", r.name)
        self.assertEqual("Z", r.end)

        list_field_one = r.list_field[0]
        self.assertEqual("A", list_field_one.field_one)
        self.assertEqual(5, list_field_one.field_two)

        list_field_two = r.list_field[1]
        self.assertEqual("B", list_field_two.field_one)
        self.assertEqual(10, list_field_two.field_two)

        self.assertEqual("XXA05B10Z", r.to_record())

    def test_from_record_raises_value_error_when_fixed_width_string_less_than_record_length(self):
        with self.assertRaises(ValueError) as e:
            RecordOne.from_record("test 000050")
        self.assertEqual("Fixed width record length is 11 but should be 12.", e.exception.message)

    def test_from_record_raises_value_error_when_fixed_width_string_greater_than_record_length(self):
        with self.assertRaises(ValueError) as e:
            RecordOne.from_record("test 00005000")
        self.assertEqual("Fixed width record length is 13 but should be 12.", e.exception.message)

    def test_truncates_each_field_when_auto_truncate(self):
        class TruncRecord(fixedwidth.Record):
            auto_truncate=True

            char = fields.StringField(length=2)
            integer = fields.IntegerField(length=3)

        r = TruncRecord(char="Too long", integer=12345)
        record = r.to_record()
        self.assertEqual("To123", record)

class FixedWidthFieldTests(TestCase):

    def get_field(self, auto_truncate=False, **kwargs):
        f = fields.FixedWidthField(**kwargs)
        f.attname = "field_one" # fake these two. Record sets them on instantiation
        f.auto_truncate = auto_truncate
        return f

    def test_raises_field_length_error_when_value_is_longer_than_allowed(self):
        f = self.get_field(length=5)
        with self.assertRaises(fields.FieldLengthError) as e:
            f.get_record_value("This is a long")

        msg = "'field_one' value 'This is a long' is longer than 5 chars."
        self.assertEqual(msg, e.exception.message)

    def test_truncates_field_length_when_has_auto_truncate_on(self):
        f = self.get_field(auto_truncate=True, length=5)
        val = f.get_record_value("This is too long")
        self.assertEqual("This ", val)

    def test_get_default_returns_none_when_no_default(self):
        field = fields.FixedWidthField(length=5)
        self.assertEqual(None, field.get_default())

    def test_get_default_returns_default_when_present(self):
        field = fields.FixedWidthField(length=5, default="ABC")
        self.assertEqual("ABC", field.get_default())

    def test_get_default_returns_called_default_when_callable(self):
        def get_default():
            return "Default Value"

        field = fields.FixedWidthField(length=15, default=get_default)
        self.assertEqual("Default Value", field.get_default())

    def test_to_record_returns_string_of_value_padded_to_length(self):
        field = fields.FixedWidthField(length=5)
        self.assertEqual("ALM  ", field.to_record("ALM"))

    def test_to_record_returns_padded_blank_string_when_value_is_None(self):
        field = fields.FixedWidthField(length=5)
        self.assertEqual("     ", field.to_record(None))

    def test_to_python_turns_value_to_string(self):
        field = fields.FixedWidthField(length=5)
        python_val = field.to_python(10)
        self.assertEqual("10", python_val)
        self.assertIsInstance(python_val, str)

    def test_to_python_strips_extra_padding_from_right(self):
        field = fields.FixedWidthField(length=5)
        self.assertEqual("AA", field.to_python("AA   "))

    def test_to_python_returns_none_when_value_is_none(self):
        field = fields.FixedWidthField(length=5)
        self.assertEqual(None, field.to_python(None))


class StringFieldTests(TestCase):

    def test_to_record_returns_string_of_value_padded_to_length(self):
        field = fields.StringField(length=5)
        self.assertEqual("AA   ", field.to_record("AA"))

    def test_to_record_returns_padded_blank_string_when_value_is_None(self):
        field = fields.StringField(length=5)
        self.assertEqual("     ", field.to_record(None))

    def test_to_python_turns_value_to_string(self):
        field = fields.StringField(length=5)
        python_val = field.to_python(10)
        self.assertEqual("10", python_val)
        self.assertIsInstance(python_val, str)

    def test_to_python_returns_none_when_value_is_none(self):
        field = fields.StringField(length=5)
        self.assertEqual(None, field.to_python(None))

    def test_to_python_returns_empty_string_when_received_all_blanks(self):
        field = fields.StringField(length=5)
        self.assertEqual("", field.to_python("     "))


class IntegerFieldTests(TestCase):

    def test_to_record_returns_string_of_value_padded_to_length(self):
        field = fields.IntegerField(length=5)
        self.assertEqual("00012", field.to_record(12))

    def test_to_record_returns_padded_zeros_when_value_is_None(self):
        field = fields.IntegerField(length=5)
        self.assertEqual("00000", field.to_record(None))

    def test_to_python_turns_value_to_int(self):
        field = fields.IntegerField(length=5)
        python_val = field.to_python("10")
        self.assertEqual(10, python_val)
        self.assertIsInstance(python_val, int)

    def test_to_python_returns_none_when_value_is_none(self):
        field = fields.IntegerField(length=5)
        self.assertEqual(None, field.to_python(None))

    def test_to_python_returns_none_when_value_is_empty_string(self):
        field = fields.IntegerField(length=5)
        self.assertEqual(None, field.to_python("     "))

    def test_to_python_raises_value_error_when_value_given_has_characters(self):
        field = fields.IntegerField(length=5)
        with self.assertRaises(ValueError):
            self.assertEqual(None, field.to_python("0001A"))


class DecimalFieldTests(TestCase):

    def test_to_record_returns_string_value_padded_to_length(self):
        field = fields.DecimalField(length=6, decimals=1)
        self.assertEqual("0010.0", field.to_record(10))

    def test_to_record_returns_padded_zeros_when_value_is_None(self):
        field = fields.DecimalField(length=7, decimals=2)
        self.assertEqual("0000.00", field.to_record(None))

    def test_to_python_turns_value_to_float(self):
        field = fields.DecimalField(length=5)
        python_val = field.to_python("10.12")
        self.assertEqual(10.12, python_val)
        self.assertIsInstance(python_val, float)

    def test_to_python_returns_none_when_value_is_none(self):
        field = fields.DecimalField(length=5)
        self.assertEqual(None, field.to_python(None))

    def test_to_python_returns_none_when_value_is_empty_string(self):
        field = fields.DecimalField(length=5)
        self.assertEqual(None, field.to_python("     "))

    def test_to_python_raises_value_error_when_value_given_has_characters(self):
        field = fields.DecimalField(length=5)
        with self.assertRaises(ValueError):
            self.assertEqual(None, field.to_python("0001A"))


class DateFieldTests(TestCase):

    def test_to_record_returns_formatted_date(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        self.assertEqual("20110831", field.to_record(date(2011,8,31)))

    def test_to_record_returns_padded_spaces_when_value_is_None(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        self.assertEqual("        ", field.to_record(None))

    def test_to_record_returns_padded_spaces_when_value_is_empty_string(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        self.assertEqual("        ", field.to_record(''))

    def test_to_python_turns_string_to_date_field(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        expected_date = date(2011, 8, 31)
        python_val = field.to_python("20110831")
        self.assertEqual(expected_date, python_val)
        self.assertIsInstance(python_val, date)

    def test_to_python_returns_none_when_no_value_given(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        self.assertEqual(None, field.to_python(None))

    def test_to_python_returns_none_when_empty_string_given(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        self.assertEqual(None, field.to_python("        "))

    def test_to_python_returns_date_when_already_date(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        expected_date = date(2011,8,31)
        python_val = field.to_python(expected_date)
        self.assertEqual(expected_date, python_val)
        self.assertIsInstance(python_val, date)

    def test_to_python_returns_date_when_given_datetime(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        expected_date = datetime(2011,8,31)
        python_val = field.to_python(expected_date)
        self.assertEqual(expected_date.date(), python_val)
        self.assertIsInstance(python_val, date)

    def test_to_python_raises_value_error_when_value_given_has_characters(self):
        field = fields.DateField(length=5)
        with self.assertRaises(ValueError):
            self.assertEqual(None, field.to_python("0001A"))

    def test_datefield_should_allow_string_default_date(self):
        class TestRecord(fixedwidth.Record):
            field_one = fields.DateField(length=10, format="%m/%d/%Y", default="08/01/2001")

        c = TestRecord()
        self.assertEqual(date(2001, 8, 1), c.field_one)

    def test_datefield_should_allow_date_default_date(self):
        default_date = date(2001,1,1)

        class TestRecord(fixedwidth.Record):
            field_one = fields.DateField(length=10, format="%m/%d/%Y", default=default_date)

        c = TestRecord()
        self.assertEqual(date(2001, 1, 1), c.field_one)

    def test_datefield_should_allow_callable_date(self):
        def get_date():
            return date(2000,1,1)

        class TestRecord(fixedwidth.Record):
            field_one = fields.DateField(length=10, format="%m/%d/%Y", default=get_date)

        c = TestRecord()
        self.assertEqual(date(2000, 1, 1), c.field_one)


class FragmentFieldTests(TestCase):

    def test_to_python_returns_record_instance_when_none_given(self):
        f = fields.FragmentField(record=RecordOne)
        python_val = f.to_python(None)
        self.assertIsInstance(python_val, RecordOne)

    def test_to_python_returns_record_instance_when_given_string(self):
        f = fields.FragmentField(record=RecordOne)
        python_val = f.to_python('AAAAA1111111')
        self.assertIsInstance(python_val, RecordOne)

    def test_to_python_returns_record_instance_when_given_instance(self):
        record = RecordOne()
        f = fields.FragmentField(record=RecordOne)
        python_val = f.to_python(record)
        self.assertIsInstance(python_val, RecordOne)

    def test_to_python_raises_type_error_when_receives_bad_input(self):
        f = fields.FragmentField(record=RecordOne)
        with self.assertRaises(TypeError) as ctx:
            f.to_python(222)
        self.assertEqual("Redefined field must be a string or RecordOne instance.", ctx.exception.message)

    def test_to_record_returns_string_of_record_when_explicitly_set(self):
        r = RecordThree(other_field="ZZZ")
        self.assertEqual("AA   0000000ZZZ", r.to_record())

    def test_to_record_returns_string_of_record_when_has_explicit_values(self):
        my_record = RecordOne(field_one="aa", field_two=999)
        r = RecordThree(frag=my_record, other_field="ZZZ")
        self.assertEqual("aa   0000999ZZZ", r.to_record())

    def test_creates_record_object_properly_from_string(self):
        r = RecordThree.from_record("aaaaa9999999ZZZ")
        self.assertEqual("aaaaa", r.sub_field.field_one)
        self.assertEqual(9999999, r.sub_field.field_two)
        self.assertEqual("ZZZ", r.other_field)
        self.assertIsInstance(r.sub_field, RecordOne)

    def test_creates_record_object_properly_from_string(self):
        r = RecordThree()
        r.frag.field_one = "aaa"
        r.frag.field_two = 999

        self.assertEqual("aaa  0000999BBB", r.to_record())


class ListFieldTests(TestCase):

    def test_to_python_returns_list_of_records(self):
        f = fields.ListField(record=RecordOne, length=2)
        records = [RecordOne(), RecordOne()]
        python_val = f.to_python(records)
        self.assertEqual(records, python_val)
        self.assertIsInstance(python_val, list)

    def test_to_python_returns_list_of_records_when_given_string(self):
        f = fields.ListField(record=RecordOne, length=2)

        records = f.to_python("ABCDE1111111ZYXWV9999999")
        self.assertEqual(len(records), 2)

        record_one = records[0]
        self.assertEqual("ABCDE", record_one.field_one)
        self.assertEqual(1111111, record_one.field_two)

        record_two = records[1]
        self.assertEqual("ZYXWV", record_two.field_one)
        self.assertEqual(9999999, record_two.field_two)

    def test_to_python_raises_type_error_when_not_given_record_class_instances(self):
        f = fields.ListField(record=RecordOne, length=2)
        records = [RecordOne(), fixedwidth.Record()]

        with self.assertRaises(TypeError) as e:
            f.to_python(records)
        self.assertEqual("List field must contain instances of 'RecordOne'.", e.exception.message)

    def test_defaults_field_to_empty_list_when_no_records_initially_given(self):
        class TestRecord(fixedwidth.Record):
            field_one = fields.ListField(RecordOne, length=2)

        r = TestRecord()
        self.assertEqual([], r.field_one)

    def test_to_record_returns_string_of_all_records_from_list_field(self):
        class TestRecord(fixedwidth.Record):
            list_field = fields.ListField(RecordOne, length=2)

        record_one = RecordOne(field_one="AAAAA", field_two=1111111)
        record_two = RecordOne(field_one="BBBBB", field_two=2222222)
        r = TestRecord(list_field=[record_one, record_two])

        self.assertEqual("AAAAA1111111BBBBB2222222", r.to_record())

    def test_to_record_pads_with_empty_records_when_not_all_records_used(self):
        class ListRecord(fixedwidth.Record):
            field_one = fields.StringField(length=5)
            field_two = fields.IntegerField(length=7)

        class TestRecord(fixedwidth.Record):
            list_field = fields.ListField(ListRecord, length=2)

        record_one = ListRecord(field_one="AAAAA", field_two=1111111)
        r = TestRecord(list_field=[record_one])

        self.assertEqual("AAAAA1111111     0000000", r.to_record())

    def test_check_record_length_returns_field_length_error_when_too_many_records_used(self):

        list_field = fields.ListField(RecordOne, length=2)
        list_field.attname = "list_field" # fake for test... fixedwidth sets this on instantiation

        record_one = RecordOne(field_one="AAAAA", field_two=1111111)
        record_two = RecordOne(field_one="BBBBB", field_two=2222222)
        record_three = RecordOne(field_one="CCCCC", field_two=3333333)

        records = [record_one, record_two, record_three]
        with self.assertRaises(fields.FieldLengthError) as e:
            list_field._check_record_length(''.join(r.to_record() for r in records))

        self.assertEqual("'list_field' contains 3 records but can only have 2.", e.exception.message)
